@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996-2000,2002 Michael R. Elkins <me@mutt.org>
+ * Copyright (C) 1996-2000,2002,2010-2011 Michael R. Elkins <me@mutt.org>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -325,7 +325,11 @@ static char *get_func (const struct binding_t *bindings, int op)
   return NULL;
 }
 
-static void push_string (char *s)
+/* Parses s for <function> syntax and adds the whole sequence to
+ * either the macro or unget buffer.  This function is invoked by the next
+ * two defines below.
+ */
+static void generic_tokenize_push_string (char *s, void (*generic_push) (int, int))
 {
   char *pp, *p = s + mutt_strlen (s) - 1;
   size_t l;
@@ -343,7 +347,7 @@ static void push_string (char *s)
       {
 	if ((i = parse_fkey (pp)) > 0)
 	{
-	  mutt_ungetch (KEY_F (i), 0);
+	  generic_push (KEY_F (i), 0);
 	  p = pp - 1;
 	  continue;
 	}
@@ -357,7 +361,7 @@ static void push_string (char *s)
 	if (KeyNames[i].name)
 	{
 	  /* found a match */
-	  mutt_ungetch (KeyNames[i].value, 0);
+	  generic_push (KeyNames[i].value, 0);
 	  p = pp - 1;
 	  continue;
 	}
@@ -377,24 +381,29 @@ static void push_string (char *s)
 
 	if (op != OP_NULL)
 	{
-	  mutt_ungetch (0, op);
+	  generic_push (0, op);
 	  p = pp - 1;
 	  continue;
 	}
       }
     }
-    mutt_ungetch ((unsigned char)*p--, 0);	/* independent 8 bits chars */
+    generic_push ((unsigned char)*p--, 0);	/* independent 8 bits chars */
   }
 }
 
+/* This should be used for macros, push, and exec commands only. */
+#define tokenize_push_macro_string(s) generic_tokenize_push_string (s, mutt_push_macro_event)
+/* This should be used for other unget operations. */
+#define tokenize_unget_string(s) generic_tokenize_push_string (s, mutt_unget_event)
+
 static int retry_generic (int menu, keycode_t *keys, int keyslen, int lastkey)
 {
   if (menu != MENU_EDITOR && menu != MENU_GENERIC && menu != MENU_PAGER)
   {
     if (lastkey)
-      mutt_ungetch (lastkey, 0);
+      mutt_unget_event (lastkey, 0);
     for (; keyslen; keyslen--)
-      mutt_ungetch (keys[keyslen - 1], 0);
+      mutt_unget_event (keys[keyslen - 1], 0);
     return (km_dokey (MENU_GENERIC));
   }
   if (menu != MENU_EDITOR)
@@ -495,11 +504,9 @@ int km_dokey (int menu)
 	  func = get_func (bindings, tmp.op);
 	  if (func)
 	  {
-	    /* careful not to feed the <..> as one token. otherwise 
-	    * push_string() will push the bogus op right back! */
-	    mutt_ungetch ('>', 0);
-	    push_string (func);
-	    mutt_ungetch ('<', 0);
+	    mutt_unget_event ('>', 0);
+	    mutt_unget_string (func);
+	    mutt_unget_event ('<', 0);
 	    break;
 	  }
 	}
@@ -526,6 +533,12 @@ int km_dokey (int menu)
       if (map->op != OP_MACRO)
 	return map->op;
 
+      if (option (OPTIGNOREMACROEVENTS))
+      {
+	mutt_error _("Macros are currently disabled.");
+	return -1;
+      }
+
       if (n++ == 10)
       {
 	mutt_flushinp ();
@@ -533,7 +546,7 @@ int km_dokey (int menu)
 	return -1;
       }
 
-      push_string (map->macro);
+      tokenize_push_macro_string (map->macro);
       map = Keymaps[menu];
       pos = 0;
     }
@@ -835,7 +848,7 @@ void km_error_key (int menu)
   }
 
   /* make sure the key is really the help key in this menu */
-  push_string (buf);
+  tokenize_unget_string (buf);
   if (km_dokey (menu) != OP_HELP)
   {
     mutt_error _("Key is not bound.");
@@ -857,7 +870,7 @@ int mutt_parse_push (BUFFER *buf, BUFFER *s, unsigned long data, BUFFER *err)
     r = -1;
   }
   else
-    push_string (buf->data);
+    tokenize_push_macro_string (buf->data);
   return (r);
 }
 
@@ -1107,7 +1120,7 @@ int mutt_parse_exec (BUFFER *buf, BUFFER *s, unsigned long data, BUFFER *err)
   while(MoreArgs(s) && nops < sizeof(ops)/sizeof(ops[0]));
 
   while(nops)
-    mutt_ungetch(0, ops[--nops]);
+    mutt_push_macro_event (0, ops[--nops]);
 
   return 0;
 }
