@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996-2000,2002 Michael R. Elkins <me@mutt.org>
+ * Copyright (C) 1996-2000,2002,2012 Michael R. Elkins <me@mutt.org>
  *
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -25,13 +25,6 @@
 #include "mutt_menu.h"
 #include "mbyte.h"
 
-#include <string.h>
-#include <stdlib.h>
-
-extern int Charset_is_utf8; /* FIXME: bad modularisation */
-
-extern size_t UngetCount;
-
 char* SearchBuffers[MENU_MAX];
 
 static void print_enriched_string (int attr, unsigned char *s, int do_color)
@@ -55,58 +48,93 @@ static void print_enriched_string (int attr, unsigned char *s, int do_color)
 	  case M_TREE_LLCORNER:
 	    if (option (OPTASCIICHARS))
 	      addch ('`');
+#ifdef WACS_LLCORNER
+	    else
+	      add_wch(WACS_LLCORNER);
+#else
 	    else if (Charset_is_utf8)
 	      addstr ("\342\224\224"); /* WACS_LLCORNER */
 	    else
 	      addch (ACS_LLCORNER);
+#endif
 	    break;
 	  case M_TREE_ULCORNER:
 	    if (option (OPTASCIICHARS))
 	      addch (',');
+#ifdef WACS_ULCORNER
+	    else
+	      add_wch(WACS_ULCORNER);
+#else
 	    else if (Charset_is_utf8)
 	      addstr ("\342\224\214"); /* WACS_ULCORNER */
 	    else
 	      addch (ACS_ULCORNER);
+#endif
 	    break;
 	  case M_TREE_LTEE:
 	    if (option (OPTASCIICHARS))
 	      addch ('|');
+#ifdef WACS_LTEE
+	    else
+	      add_wch(WACS_LTEE);
+#else
 	    else if (Charset_is_utf8)
 	      addstr ("\342\224\234"); /* WACS_LTEE */
 	    else
 	      addch (ACS_LTEE);
+#endif
 	    break;
 	  case M_TREE_HLINE:
 	    if (option (OPTASCIICHARS))
 	      addch ('-');
+#ifdef WACS_HLINE
+	    else
+	      add_wch(WACS_HLINE);
+#else
 	    else if (Charset_is_utf8)
 	      addstr ("\342\224\200"); /* WACS_HLINE */
 	    else
 	      addch (ACS_HLINE);
+#endif
 	    break;
 	  case M_TREE_VLINE:
 	    if (option (OPTASCIICHARS))
 	      addch ('|');
+#ifdef WACS_VLINE
+	    else
+	      add_wch(WACS_VLINE);
+#else
 	    else if (Charset_is_utf8)
 	      addstr ("\342\224\202"); /* WACS_VLINE */
 	    else
 	      addch (ACS_VLINE);
+#endif
 	    break;
 	  case M_TREE_TTEE:
 	    if (option (OPTASCIICHARS))
 	      addch ('-');
+#ifdef WACS_TTEE
+	    else
+	      add_wch(WACS_TTEE);
+#else
 	    else if (Charset_is_utf8)
 	      addstr ("\342\224\254"); /* WACS_TTEE */
 	    else
 	      addch (ACS_TTEE);
+#endif
 	    break;
 	  case M_TREE_BTEE:
 	    if (option (OPTASCIICHARS))
 	      addch ('-');
+#ifdef WACS_BTEE
+	    else
+	      add_wch(WACS_BTEE);
+#else
 	    else if (Charset_is_utf8)
 	      addstr ("\342\224\264"); /* WACS_BTEE */
 	    else
 	      addch (ACS_BTEE);
+#endif
 	    break;
 	  case M_TREE_SPACE:
 	    addch (' ');
@@ -129,7 +157,7 @@ static void print_enriched_string (int attr, unsigned char *s, int do_color)
 	}
 	s++, n--;
       }
-      if (do_color) attrset(attr);
+      if (do_color) ATTRSET(attr);
     }
     else if ((k = mbrtowc (&wc, (char *)s, n, &mbstate)) > 0)
     {
@@ -165,7 +193,7 @@ static void menu_pad_string (char *s, size_t n)
 
 void menu_redraw_full (MUTTMENU *menu)
 {
-  SETCOLOR (MT_COLOR_NORMAL);
+  NORMAL_COLOR;
   /* clear() doesn't optimize screen redraws */
   move (0, 0);
   clrtobot ();
@@ -175,7 +203,7 @@ void menu_redraw_full (MUTTMENU *menu)
     SETCOLOR (MT_COLOR_STATUS);
     move (option (OPTSTATUSONTOP) ? LINES-2 : 0, 0);
     mutt_paddstr (COLS, menu->help);
-    SETCOLOR (MT_COLOR_NORMAL);
+    NORMAL_COLOR;
     menu->offset = 1;
     menu->pagelen = LINES - 3;
   }
@@ -198,7 +226,7 @@ void menu_redraw_status (MUTTMENU *menu)
   SETCOLOR (MT_COLOR_STATUS);
   move (option (OPTSTATUSONTOP) ? 0 : LINES - 2, 0);
   mutt_paddstr (COLS, buf);
-  SETCOLOR (MT_COLOR_NORMAL);
+  NORMAL_COLOR;
   menu->redraw &= ~REDRAW_STATUS;
 }
 
@@ -206,55 +234,46 @@ void menu_redraw_index (MUTTMENU *menu)
 {
   char buf[LONG_STRING];
   int i;
+  int do_color;
+  int attr;
 
   for (i = menu->top; i < menu->top + menu->pagelen; i++)
   {
     if (i < menu->max)
     {
+      attr = menu->color(i);
+
       menu_make_entry (buf, sizeof (buf), menu, i);
       menu_pad_string (buf, sizeof (buf));
 
-      if (option (OPTARROWCURSOR))
+      ATTRSET(attr);
+      move(i - menu->top + menu->offset, 0);
+      do_color = 1;
+
+      if (i == menu->current)
       {
-        attrset (menu->color (i));
-	CLEARLINE (i - menu->top + menu->offset);
-
-	if (i == menu->current)
-	{
-          attrset (menu->color (i));
-	  ADDCOLOR (MT_COLOR_INDICATOR);
-	  addstr ("->");
-          attrset (menu->color (i));
-	  addch (' ');
-	}
-	else
-	{
-	  attrset (menu->color (i));
-	  addstr ("   ");
-	}
-
-        print_enriched_string (menu->color(i), (unsigned char *) buf, 1);
-        SETCOLOR (MT_COLOR_NORMAL);          
+	  SETCOLOR(MT_COLOR_INDICATOR);
+	  if (option(OPTARROWCURSOR))
+	  {
+	    addstr ("->");
+	    ATTRSET(attr);
+	    addch (' ');
+	  }
+	  else
+	    do_color = 0;
       }
-      else
-      {
-        attrset (menu->color (i));
-            
-	if (i == menu->current)
-	{
-	  ADDCOLOR (MT_COLOR_INDICATOR);
-	  BKGDSET (MT_COLOR_INDICATOR);
-	}
+      else if (option(OPTARROWCURSOR))
+	addstr("   ");
 
-	CLEARLINE (i - menu->top + menu->offset);
-	print_enriched_string (menu->color(i), (unsigned char *) buf, i != menu->current);
-        SETCOLOR (MT_COLOR_NORMAL);
-        BKGDSET (MT_COLOR_NORMAL);
-      }
+      print_enriched_string (attr, (unsigned char *) buf, do_color);
     }
     else
-      CLEARLINE (i - menu->top + menu->offset);
+    {
+      NORMAL_COLOR;
+      CLEARLINE(i - menu->top + menu->offset);
+    }
   }
+  NORMAL_COLOR;
   menu->redraw = 0;
 }
 
@@ -269,37 +288,28 @@ void menu_redraw_motion (MUTTMENU *menu)
   }
   
   move (menu->oldcurrent + menu->offset - menu->top, 0);
-  SETCOLOR (MT_COLOR_NORMAL);
-  BKGDSET (MT_COLOR_NORMAL);
+  ATTRSET(menu->color (menu->oldcurrent));
 
   if (option (OPTARROWCURSOR))
   {
     /* clear the pointer */
-    attrset (menu->color (menu->oldcurrent));
     addstr ("  ");
 
     if (menu->redraw & REDRAW_MOTION_RESYNCH)
     {
-      clrtoeol ();
       menu_make_entry (buf, sizeof (buf), menu, menu->oldcurrent);
       menu_pad_string (buf, sizeof (buf));
       move (menu->oldcurrent + menu->offset - menu->top, 3);
       print_enriched_string (menu->color(menu->oldcurrent), (unsigned char *) buf, 1);
-      SETCOLOR (MT_COLOR_NORMAL);
     }
 
     /* now draw it in the new location */
-    move (menu->current + menu->offset - menu->top, 0);
-    attrset (menu->color (menu->current));
-    ADDCOLOR (MT_COLOR_INDICATOR);
-    addstr ("->");
-    SETCOLOR (MT_COLOR_NORMAL);
+    SETCOLOR(MT_COLOR_INDICATOR);
+    mvaddstr(menu->current + menu->offset - menu->top, 0, "->");
   }
   else
   {
     /* erase the current indicator */
-    attrset (menu->color (menu->oldcurrent));
-    clrtoeol ();
     menu_make_entry (buf, sizeof (buf), menu, menu->oldcurrent);
     menu_pad_string (buf, sizeof (buf));
     print_enriched_string (menu->color(menu->oldcurrent), (unsigned char *) buf, 1);
@@ -307,50 +317,36 @@ void menu_redraw_motion (MUTTMENU *menu)
     /* now draw the new one to reflect the change */
     menu_make_entry (buf, sizeof (buf), menu, menu->current);
     menu_pad_string (buf, sizeof (buf));
-    attrset (menu->color (menu->current));
-    ADDCOLOR (MT_COLOR_INDICATOR);
-    BKGDSET (MT_COLOR_INDICATOR);
-    CLEARLINE (menu->current - menu->top + menu->offset);
+    SETCOLOR(MT_COLOR_INDICATOR);
+    move(menu->current - menu->top + menu->offset, 0);
     print_enriched_string (menu->color(menu->current), (unsigned char *) buf, 0);
-    SETCOLOR (MT_COLOR_NORMAL);
-    BKGDSET (MT_COLOR_NORMAL);
   }
   menu->redraw &= REDRAW_STATUS;
+  NORMAL_COLOR;
 }
 
 void menu_redraw_current (MUTTMENU *menu)
 {
   char buf[LONG_STRING];
+  int attr = menu->color (menu->current);
   
   move (menu->current + menu->offset - menu->top, 0);
   menu_make_entry (buf, sizeof (buf), menu, menu->current);
   menu_pad_string (buf, sizeof (buf));
 
+  SETCOLOR(MT_COLOR_INDICATOR);
   if (option (OPTARROWCURSOR))
   {
-    int attr = menu->color (menu->current);
-    attrset (attr);
-    clrtoeol ();
-    attrset (menu->color (menu->current));
-    ADDCOLOR (MT_COLOR_INDICATOR);
     addstr ("->");
-    attrset (attr);
+    ATTRSET(attr);
     addch (' ');
     menu_pad_string (buf, sizeof (buf));
-    print_enriched_string (menu->color(menu->current), (unsigned char *) buf, 1);
-    SETCOLOR (MT_COLOR_NORMAL);
+    print_enriched_string (attr, (unsigned char *) buf, 1);
   }
   else
-  {
-    attrset (menu->color (menu->current));
-    ADDCOLOR (MT_COLOR_INDICATOR);
-    BKGDSET (MT_COLOR_INDICATOR);
-    clrtoeol ();
-    print_enriched_string (menu->color(menu->current), (unsigned char *) buf, 0);
-    SETCOLOR (MT_COLOR_NORMAL);
-    BKGDSET (MT_COLOR_NORMAL);
-  }
+    print_enriched_string (attr, (unsigned char *) buf, 0);
   menu->redraw &= REDRAW_STATUS;
+  NORMAL_COLOR;
 }
 
 static void menu_redraw_prompt (MUTTMENU *menu)
@@ -366,7 +362,6 @@ static void menu_redraw_prompt (MUTTMENU *menu)
     if (*Errorbuf)
       mutt_clear_error ();
 
-    SETCOLOR (MT_COLOR_NORMAL);
     mvaddstr (LINES - 1, 0, menu->prompt);
     clrtoeol ();
   }
@@ -418,7 +413,7 @@ void menu_jump (MUTTMENU *menu)
 
   if (menu->max)
   {
-    mutt_ungetch (LastKey, 0);
+    mutt_unget_event (LastKey, 0);
     buf[0] = 0;
     if (mutt_get_field (_("Jump to: "), buf, sizeof (buf), 0) == 0 && buf[0])
     {
@@ -687,6 +682,9 @@ MUTTMENU *mutt_new_menu (int menu)
 {
   MUTTMENU *p = (MUTTMENU *) safe_calloc (1, sizeof (MUTTMENU));
 
+  if ((menu < 0) || (menu >= MENU_MAX))
+    menu = MENU_GENERIC;
+
   p->menu = menu;
   p->current = 0;
   p->top = 0;
@@ -817,7 +815,7 @@ static int menu_dialog_dokey (MUTTMENU *menu, int *ip)
   }
   else
   {
-    mutt_ungetch (ch.op ? 0 : ch.ch, ch.op ? ch.op : 0);
+    mutt_unget_event (ch.op ? 0 : ch.ch, ch.op ? ch.op : 0);
     return -1;
   }
 }
@@ -905,12 +903,7 @@ int mutt_menuLoop (MUTTMENU *menu)
       }
       else /* None tagged, OP_TAG_PREFIX_COND */
       {
-	event_t tmp;
-	while(UngetCount>0)
-	{
-	  tmp=mutt_getch();
-	  if(tmp.op==OP_END_COND)break;
-	}
+	mutt_flush_macro_to_endcond ();
 	mutt_message _("Nothing to do.");
 	i = -1;
       }
