@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996-2000 Michael R. Elkins <me@mutt.org>
+ * Copyright (C) 1996-2000,2010 Michael R. Elkins <me@mutt.org>
  * Copyright (C) 2000-2002 Edmund Grimley Evans <edmundo@rano.org>
  * 
  *     This program is free software; you can redistribute it and/or modify
@@ -438,7 +438,7 @@ static int rfc2047_encode (ICONV_CONST char *d, size_t dlen, int col,
       if (!t0) t0 = t;
       t1 = t;
     }
-    else if (specials && strchr (specials, *t))
+    else if (specials && *t && strchr (specials, *t))
     {
       if (!s0) s0 = t;
       s1 = t;
@@ -740,7 +740,7 @@ static const char *find_encoded_word (const char *s, const char **x)
 	 0x20 < *q && *q < 0x7f && !strchr ("()<>@,;:\"/[]?.=", *q);
 	 q++)
       ;
-    if (q[0] != '?' || !strchr ("BbQq", q[1]) || q[2] != '?')
+    if (q[0] != '?' || q[1] == '\0' || !strchr ("BbQq", q[1]) || q[2] != '?')
       continue;
     /* non-strict check since many MUAs will not encode spaces and question marks */
     for (q = q + 3; 0x20 <= *q && *q < 0x7f && (*q != '?' || q[1] != '='); q++)
